#include <LiquidCrystal.h>



#define LED_COUNT 8

//setup pins
const byte redSliderPin   = 0;
const byte greenSliderPin = 1;
const byte blueSliderPin  = 2;
const byte whiteSliderPin = 3;

const byte valuePotPin    = 4;
const byte directionPotPin  = 5;
const byte magnitudePotPin  = 6;
const byte randomPotPin = 7;

byte buttonPins[]= {40,41,42,43};
byte buttonState[]= {0,0,0,0};
byte buttonLastState[]= {0,0,0,0};
long buttonLastDebounce;
byte buttonJustPressed[] = {0,0,0,0};
byte buttonPressed[] = {0,0,0,0};
byte buttonA = 1;
byte buttonB = 1;
byte buttonAMax = 8;
byte buttonBMax = 8;

long programMillis;

typedef struct {
  byte r;
  byte g;
  byte b;
  byte w;
} RGBW;

RGBW leds[8]; 

LiquidCrystal lcd(2, 3, 4, 5, 6, 7);

// Init our Input Vars
byte redSlider;
byte greenSlider;
byte blueSlider;
byte whiteSlider;

byte valuePot;
float directionPot;
float magnitudePot;
float randomPot;

int lastProgramA = 0;
boolean runSetup = false;



  
void readValues()
{
  redSlider   = analogRead(redSliderPin)/4; 
  greenSlider = analogRead(greenSliderPin)/4;
  blueSlider  = analogRead(blueSliderPin)/4;
  whiteSlider = analogRead(whiteSliderPin)/4;
  
  valuePot     = analogRead(valuePotPin)/4; 
  directionPot = (float) (analogRead(directionPotPin)-512)/128;
  magnitudePot = (float) analogRead(magnitudePotPin)/256;
  randomPot    = (float) analogRead(randomPotPin);
  
} 
//keep code cleaner and seperate out some things
#include "color_fns.h"
#include "led_fades.h"
#include "color_programs.h"


//boolean sendData = false;

boolean readSerial()
{
  if(Serial3.available() && (Serial3.read()==65)) {
    return true;
  }

  Serial3.flush();
  return false;
}

void sendSerial() 
{

  
    Serial3.print(buttonB, BYTE);
    Serial3.print(valuePot, BYTE);
    for(int i=0; i<LED_COUNT; i++) {
      Serial3.print(leds[i].r, BYTE);
      Serial3.print(leds[i].g, BYTE);
      Serial3.print(leds[i].b, BYTE);
      Serial3.print(leds[i].w, BYTE);
   /*
      Serial.print(leds[i].r, DEC);
      Serial.print(leds[i].g, DEC);
      Serial.print(leds[i].b, DEC);
      Serial.print(leds[i].w, DEC);
      Serial.print(',', BYTE);//*/
    }
   //Serial.print(10, BYTE);
  

}
void readButtons(){
// based on code from http://www.adafruit.com/blog/2009/10/20/example-code-for-multi-button-checker-with-debouncing/
  int debounceDelay = 10;
  
  if (millis() < buttonLastDebounce) {
     // we wrapped around, lets just try again
     buttonLastDebounce = millis();
  }
  if ((buttonLastDebounce + debounceDelay) > millis()) {
    // not enough time has passed to debounce
    return;
  }

  for(byte i = 0; i<sizeof(buttonPins); i++) {
    buttonJustPressed[i] = 0;       // when we start, we clear out the "just" indicators


    buttonState[i] = digitalRead(buttonPins[i]);   // read the button


    if (buttonState[i] == buttonLastState[i]) {
      if ((buttonPressed[i] == LOW) && (buttonState[i] == LOW)) {
          // just pressed
          buttonJustPressed[i] = 1;
      }
    
      buttonPressed[i] = !buttonState[i];  // remember, digital HIGH means NOT pressed
    }
    //Serial.println(pressed[index], DEC);
    buttonLastState[i] = buttonState[i];   // keep a running tally of the buttons
  }
  
  if(buttonJustPressed[0] == 1 && buttonA < buttonAMax) {
    buttonA++;
  }
  if(buttonJustPressed[1] == 1 && buttonA > 1) {
    buttonA--;
  }
  if(buttonJustPressed[2] == 1 && buttonB < buttonBMax) {
    buttonB++;
  }
  if(buttonJustPressed[3] == 1 && buttonB > 1) {
    buttonB--;
  }
}

void runProgram() {
 if(buttonA != lastProgramA) {
      runSetup = true;
      lastProgramA = buttonA;
  }
  lcd.setCursor(3, 0);
  switch (buttonA) {
    case 1:
        solidColor();
      
      break;
    case 2:
       blinkI();
      break;
    case 3:  
      rainbow();
      break;
    case 4:  
      shiftInColor();
      break; 
      
    case 5:
      randomWhiteFlash();
      break;
    case 6:
      fadeI();
      break;
    case 7:
      fadeSat();
      break;
    default: 
      
    //          "              "
      lcd.print("Blank         ");
     break;
  }
  lcd.setCursor(3, 1);
switch (buttonB) {
    case 1:
       //glow where people are
      lcd.print("Light Only    ");
      break;
    case 2:
      //glow white
      lcd.print("Glow White     ");
      break;
    case 3:  
       //shift hue
       lcd.print("Shift Hue    ");
      break;
    case 4:  
        //drop sateration
      lcd.print("Drop Saturation");
      break; 
      
    case 5:
        //increase brightness
      lcd.print("Boost Bright    ");
      break;
     
      
    default:  
    lcd.print("None          ");  
     break;
  }   


  

}
long loopTimer;
void setup()
{
  Serial3.begin(115200);
  Serial.begin(115200);
 
  // set up the LCD's number of rows and columns: 
  lcd.begin(16, 2);
   lcd.clear();
  lcd.print("A");
 // lcd.print(buttonATitle);
  lcd.setCursor(0, 1);
  lcd.print("B");
 // lcd.print(buttonB,DEC);
 sendSerial();
 loopTimer = millis();
}


void loop()
{


   
  readButtons();
   
 if(loopTimer+33 < millis()){   
  if(readSerial()){
  loopTimer = millis();
  
  readValues();
  runProgram();
  lcd.setCursor(1, 0);
  lcd.print(buttonA,DEC);
  lcd.setCursor(1, 1);
  lcd.print(buttonB,DEC);
  LED_updateFades();
  sendSerial();
  
//  lcd.print('   ',BYTE);
  }}
   
  
 /*
 //debug slider
   lcd.setCursor(0, 0);
  lcd.print("R: ");
  lcd.print(redSlider,DEC);
  lcd.print('  ',BYTE);

  lcd.setCursor(8, 0);
  lcd.print("G: ");
  lcd.print(greenSlider,DEC);
  lcd.print('  ',BYTE);
  
  lcd.setCursor(0, 1);
  lcd.print("B: ");
  lcd.print(blueSlider,DEC);
  lcd.print('  ',BYTE);
  
  lcd.setCursor(8, 1);
  lcd.print("W: ");
  lcd.print(whiteSlider,DEC);
  lcd.print('  ',BYTE);
}
 */
 /*
  //debug rheostats
  lcd.setCursor(0, 0);
  lcd.print("1: ");
  lcd.print(valuePot,DEC);
  lcd.print('    ',BYTE);

  lcd.setCursor(8, 0);
  lcd.print("2: ");
  lcd.print(directionPot);
  lcd.print('    ',BYTE);
  
  lcd.setCursor(0, 1);
  lcd.print("3: ");
  lcd.print(magnitudePot);
  lcd.print('    ',BYTE);
  
  lcd.setCursor(8, 1);
  lcd.print("4: ");
  lcd.print(randomnessPot);
  lcd.print('    ',BYTE);
 */
 

}

